classdef Result < handle
    % RESULT Result obtained from a simulation with Rotare.
    %   This class defines the Result object that can be used to store the solution an entire Rotare
    %   simulation.
    %
    % Notes:
    %   - Almost all properties are private to prevent accidental overwriting.
    %
    % -----
    %
    % Result properties:
    %   leishman - Results obtained with the leishman solver
    %   indfact  - Results obtained with the indFact solver
    %   indvel   - Results obtained with the indVel solver
    %   stahlhut - Results obtained with the stalhlut solver
    %   operPts  - Table with the operating points for each individual result
    %   ModExt   - Modelling enstension options (losses, etc)
    %
    %
    % Result methods:
    %   save - Save the Result object to file
    %
    % Result constructor:
    %   Result = Result() creates an empty object.
    %
    % Constructor inputs:
    %
    % See also: Rotor, rotare, template, af_tools.Airfoil.
    %
    % <a href="https:/gitlab.uliege.be/rotare/documentation">Complete documentation (online)</a>

    % ----------------------------------------------------------------------------------------------
    % FIXME: Make plot methods to compare two results easily
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2023 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/rotare/rotare
    % Docs: https://gitlab.uliege.be/rotare/documentation
    % Issues: https://gitlab.uliege.be/rotare/rotare/-/issues
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Base properties of the elements
    properties
        leishman (1, :) OperRotor  % Results obtained with the leishman solver
        indfact (1, :) OperRotor   % Results obtained with the indFact solver
        indvel (1, :) OperRotor    % Results obtained with the indVel solver
        stahlhut (1, :) OperRotor  % Results obtained with the stalhlut solver
        operPts table              % Table of operating points for each individual result
        ModExt (1, 1) struct       % Modelling enstension options (losses, etc)
    end

    properties (Dependent)
        cT
        cP
        cQ
    end

    properties (Constant, Hidden)
        SOLVERS = {'leishman', 'indfact', 'indvel', 'stahlhut'} % List of allowed solvers
    end

    methods

        function self = Result()
            % Result Constructor.
            %   Creates the Result object.
            %
            % Currently empty on purpose. Results are to be assigned manually.
        end

        % ---------------------------------------------
        % Set methods
        function set.operPts(self, val)
            % Note: A bug in Matlab prevents the user of variable names when  size checks are added
            % in the property definition. https://stackoverflow.com/q/48423003
            self.operPts = val;
            self.operPts.Properties.VariableNames = {'Alt', 'V_ax', 'RPM', 'Coll'};
        end

        % ---------------------------------------------
        % Get methods for dependent properties
        function cT = get.cT(self)
            cT = maketable(self, 'cT');
        end

        function cQ = get.cQ(self)
            cQ = maketable(self, 'cQ');
        end

        function cP = get.cP(self)
            cP = maketable(self, 'cP');
        end

        % ---------------------------------------------
        % Other methods
        save(self, SaveOpts)  % Save the results to file
        summaryTable = summarize(self, propList) % Summarize results
    end
end

function resTable = maketable(self, prop)
    % MAKETABLE Gather properties from all solvers and all operating points in a table

    resTable = table('Size', [size(self.operPts, 1), 4], ...
                     'VariableTypes', {'double', 'double', 'double', 'double'});

    propName = cell(1, 4);
    for i = 1:length(self.SOLVERS)
        propName{i} = [prop, '_', self.SOLVERS{i}];
    end

    resTable.Properties.VariableNames = propName;

    for iOp = 1:size(self.operPts, 1)
        for iSolv = 1:length(self.SOLVERS)
            solver = self.SOLVERS{iSolv};

            if ~isempty(self.(solver))
                resTable.(iSolv)(iOp) = self.(solver)(iOp).(prop);
            else
                resTable.(iSolv)(iOp) = NaN;
            end
        end
    end

end
